function [mtropt,atropt,yopt,welfare,efmono,efcounter] = mtroptimal(My,mass,r,kappa,beta,betainvhr,cbar,e,gamma,rho,delta,revex)
% compute optimal marginal tax rates, optimal average tax rates, income
% vector at the optimum, welfare level, test for monotonicity, test for
% convergence

% initial welfare weights
g = 1/(1+mass+0.05)*ones(numel(My),1);
gtilde = ones(numel(g),1); % average welfare weigth above type

% initialize mtrs
mtropt = zeros(numel(My),1);

% set convergence criterion for welfare weights
%gtolrel = 0.01;
%gdiffrelmax = gtolrel + 1;
gtol = 10^(-5);
gdiffmax = gtol + 1;

% set convergence speed
if (r > 5) && (rho > -gamma) && (mass == 0)
    up = 0.01;
else
    up = 0.2;
end


% initialize counter
counter = 0;
printcounter = 0; % to display convergence criteria during iterations
countermax = 2000;

% iteration
while (gdiffmax > gtol) && (counter < countermax) %gdiffrelmax > gtolrel
    
    % compute \tilde{g} (average welfare weight of types above theta)
%     for j = 1:numel(g)
%         if j==1
%             gtilde(j) = 1;
%         elseif j < numel(g)
%             gtilde(j) = trapz(My(j:end),g(j:end))/(My(end)-My(j));
%         else
%             gtilde(j) = g(j);
%         end
%     end
    % compute average welfare weight above type
    for j = 1:numel(g)
        if j < numel(g)
            gtilde(j) = trapz(My(j:end),g(j:end))/(My(end)-My(j));
        else
            gtilde(j) = g(j);
        end
    end

    % compute optimal mtrs, based on formula in the paper
    mtroptratio = max(-0.9,(e+1)/e*betainvhr.*(1-gtilde)+gamma*(1-g)+rho*(1-g));

    mtroptnew = mtroptratio./(1+mtroptratio);
    mtrdiffmax = max(abs(mtroptnew-mtropt)); % check max difference between new and old mtrs
    mtropt = mtroptnew;
    %min(mtroptnew,0.99999);
    %mtropt(mtropt >= 1) = -0.1;

    % compute income vector at optimal mtrs for endogenous tech planner
    % first compute labor supply, then obtain income via worker foc
    lopt = laborinput(mtropt,My,kappa,beta,cbar,e,gamma,rho,delta);
%     ltilde = (1-mtropt).^(e/(1-e*(gamma+rho))).*beta.^(e/(1-e*(gamma+rho)));%*cbar^(e*(1+gamma)/(delta*(1-e*(gamma+rho))));
%     lscale = fsolve(@(scale) wageequation(1-mtropt,My,kappa,beta,scale,ltilde,cbar,e,gamma,rho,delta),1);
%     lopt = ltilde*lscale;
    yopt = lopt.^((e+1)/e)./(1-mtropt);

    % compute tax liabilities without demogrant
    toptaux = cumtrapz(yopt,mtropt);

    % compute demogrant
    demo = (revex-trapz(My,toptaux))*1/(1+mass);

    % compute consumption and utility
    copt = yopt - toptaux - demo;
    uopt = copt - e/(e+1)*lopt.^((e+1)/e);

    % compute new welfare weights and differences
    %gnew = uopt.^(-r)/trapz(My,uopt.^(-r));
    gnew = uopt.^(-r)/(mass*(-demo)^(-r)+trapz(My,uopt.^(-r))); % new welfare weights
    gdiff = abs(gnew - g); % difference
    gdiffmax = max(gdiff); % max difference for comp with convergence crit.
    %gdiffrel = abs(gnew./g - 1);
    %gdiffrelmax = max(gdiffrel);

    % update welfare weights
    g = up*gnew + (1-up)*g;
    
    % advance counter and show convergence criteria
    counter = counter+1;
    if counter > printcounter
        printcounter = printcounter+50;
        gdiffmax
        mtrdiffmax
    end

end

% compute optimal average tax rates
atropt = (toptaux + demo)./yopt;

% compute welfare at the optimum
if r == 1
    welfare = exp(mass*log(-demo)+trapz(My,log(uopt)));
else
    welfare = (mass*(-demo)^(1-r)+trapz(My,uopt.^(1-r)))^(1/(1-r));
end

% exit flag for convergence
if counter < countermax
    efcounter = 1;
else
    efcounter = 0;
end

% exit flag for monotonicity condition
ytest = yopt(2:end)-yopt(1:(end-1));
if all(ytest>0)
    efmono = 1;
else
    efmono = 0;
end

end