function [mtropt,atropt,yopt,efmono,efcounter] = mtroptimalex(My,mass,r,kappa,betabar,betabarinvhr,phistar,e,gamma,revex)
% compute optimal marginal tax rates for the exogenous
% technology planner, optimal average tax rates, income vector at the
% optimum, test for monotonicity condition, test for convergence

% initial welfare weights
g = 1/(1+mass+0.05)*ones(numel(My),1);
gtilde = ones(numel(g),1);

% initial mtrs
mtropt = zeros(numel(My),1);

% set convergence criterion
%gtolrel = 0.01;
%gdiffrelmax = gtolrel + 1;
gtol = 10^(-5);
gdiffmax = gtol + 1;

% set convergence speed
up = 0.1;

% initialize counter
counter = 0;
printcounter = 0;
countermax = 1000;

% iteration
while (gdiffmax > gtol) && (counter < countermax) %gdiffrelmax > gtolrel
    
    % compute \tilde{g} (average welfare weight of types above theta)
%     for j = 1:numel(g)
%         if j==1
%             gtilde(j) = 1;
%         elseif j < numel(g)
%             gtilde(j) = trapz(My(j:end),g(j:end))/(My(end)-My(j));
%         else
%             gtilde(j) = g(j);
%         end
%     end
    % compute average welfare weight above type
    for j = 1:numel(g)
        if j < numel(g)
            gtilde(j) = trapz(My(j:end),g(j:end))/(My(end)-My(j));
        else
            gtilde(j) = g(j);
        end
    end

    % compute optimal mtrs, based on formula provided in the paper
    mtroptratio = (e+1)/e*betabarinvhr.*(1-gtilde)+gamma*(1-g);

    mtroptnew = mtroptratio./(1+mtroptratio); % mtrs
    mtrdiffmax = max(abs(mtroptnew-mtropt)); % check difference to previous iteration
    mtropt = mtroptnew;

    % compute income vector at optimal mtrs for endogenous tech planner
    % first compute labor supply, then obtain income via worker foc
    ltilde = (1-mtropt).^(e/(1-e*gamma)).*betabar.^(e/(1-e*gamma)); % proportional to labor inputs
    lscale = trapz(My,(kappa.*phistar.*ltilde).^(1+gamma))^(-e*gamma/(1+gamma)); % scaling factor for ltilde
    %lscale = fsolve(@(scale) scale*ltilde - (1-mtropt).^(e/(1-e*gamma)).*betabar.^(e/(1-e*gamma))...
    %    *trapz(My,(kappa.*phistar.*ltilde*scale).^(1+gamma))^(-e*gamma/((1+gamma)*(1-e*gamma))),1);
    lopt = ltilde*lscale; % labor inputs
    yopt = lopt.^((e+1)/e)./(1-mtropt);

    % compute tax liabilities without demogrant
    toptaux = cumtrapz(yopt,mtropt);

    % compute demogrant
    demo = (revex-trapz(My,toptaux))*1/(1+mass);

    % compute consumption and utility
    copt = yopt - toptaux - demo;
    uopt = copt - e/(e+1)*lopt.^((e+1)/e);

    % compute new welfare weights and differences
    %gnew = uopt.^(-r)/trapz(My,uopt.^(-r));
    gnew = uopt.^(-r)/(mass*(-demo)^(-r)+trapz(My,uopt.^(-r)));
    gdiff = abs(gnew - g);
    gdiffmax = max(gdiff);
    %gdiffrel = abs(gnew./g - 1);
    %gdiffrelmax = max(gdiffrel);

    % update welfare weights
    g = up*gnew + (1-up)*g;
    
    % advance counter and print convergence criteria
    counter = counter+1;
    if counter > printcounter
        printcounter = printcounter+50;
        gdiffmax
        mtrdiffmax
    end

end

% compute optimal average tax rates
atropt = (toptaux + demo)./yopt;

% exit flag for convergence
if counter < countermax
    efcounter = 1;
else
    efcounter = 0;
end

% exit flag for monotonicity condition
ytest = yopt(2:end)-yopt(1:(end-1));
if all(ytest>0)
    efmono = 1;
else
    efmono = 0;
end

end